import { AlertService, Modal, ModalService } from '@zeedhi/common';
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import { TekGrid, TekMemoryDatasource } from '@zeedhi/teknisa-components-common';

export class MonitoringCloudController {
  private monitoringModal!: Modal;

  private intervals: any = [];

  private monitoringCloudResult: any = {};

  private gridData: any = [];

  private monitoringParams: any = [];

  private id: any = '';

  public async gridMounted() {
    const grid = Metadata.getInstance<TekGrid>('monitoring-cloud-grid');

    try {
      grid.datasource.loading = true;
      this.monitoringModal = await ModalService.createFromJson('monitoringCloudModal', true);

      const reqParams = `${process.env.VUE_APP_METADATA_END_POINT}parametrizacoes`;
      const config = await Http.get(reqParams);
      this.monitoringParams = config.data.data.filter((element: any) => element.ATIVO === 'S' && element.TIPO === 'C');
      this.gridData = [];

      const lastFilter = localStorage.getItem('LAST_FILTER_MONITORING_CLOUD');
      if (lastFilter) {
        (grid.datasource as TekMemoryDatasource).dynamicFilter = JSON.parse(lastFilter);
      }
      if (!this.monitoringParams.length) {
        this.showAlert('Nenhum monitoramento ativo do tipo "Apache - Cloud" encontrado.', 'warning');
      }
      // if (this.monitoringParams.length > 0) {
      //   await Promise.all(this.monitoringParams.map(async (element: any) => {
      //     // Executa uma vez assim que abrir a tela para não ter delay do intervalo
      //     await this.execMonitoring(element, grid);

      //     // Executa os intervalos de cada monitoramento
      //     this.intervals.push(setInterval(() => {
      //       this.execMonitoring(element, grid);
      //     }, element.INTERVALO * 1000));
      //   }));
      // } else {
      //   this.showAlert('Nenhum monitoramento ativo do tipo "Apache - Cloud" encontrado.', 'warning');
      // }

      grid.datasource.loading = false;
    } catch (error) {
      grid.datasource.loading = false;
      this.showAlert(error, 'error');
    }
  }

  public async execMonitoring(element: any, grid: TekGrid) {
      const reqParams = `${process.env.VUE_APP_METADATA_END_POINT}cloudMonitoring`;
      const response = await Http.post(reqParams, element);
      this.monitoringCloudResult[element.ID] = response.data;

      const idx = this.gridData.findIndex((item: any) => item.ID === element.ID);
      const status = response.data.some((item:any) => item.STATUS === 'N') ? 'N' : 'S';
      if (idx === -1) {
        this.gridData.push({
          ID: element.ID,
          NAME: element.NOME,
          STATUS: status,
        });
      } else {
        this.gridData[idx].STATUS = status;
      }
      grid.datasource.updateData(this.gridData.sort(this.orderGridData));
  }

  private orderGridData(element1: any, element2: any) {
    if (element1.STATUS === 'N' && element2.STATUS === 'S') {
      return -1;
    }
    if (element1.STATUS === 'S' && element2.STATUS === 'N') {
      return 1;
    }
    return 0;
  }

  public async gridBeforeDestroy() {
    this.monitoringModal?.destroy();
    this.intervals.forEach((interval: any) => {
      clearInterval(interval);
    });
  }

  public async gridBeforeApplyFilter() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-cloud-grid');
    const filter = (datasource as TekMemoryDatasource).dynamicFilter;
    localStorage.setItem('LAST_FILTER_MONITORING_CLOUD', JSON.stringify(filter));
  }

  public getStatusColor({ row }: IDictionary<any>) {
    return row.STATUS === 'S' ? 'green' : 'red';
  }

  public buttonClickCloudMonitoring(monitor:any) {
    if (monitor.component.color === 'green') {
      this.showAlert('Tudo certo com esse monitoramento!', 'success');
    } else {
      this.id = monitor.row.ID;
      const config = this.monitoringParams.find((element: any) => element.ID === monitor.row.ID);
      this.monitoringModal.children[1].title = config.DESCRICAO;
      this.monitoringModal.show();
    }
  }

  public modalGridMounted() {
    const { datasource } = Metadata.getInstance<TekGrid>('monitoring-cloud-modal-grid');
    datasource.updateData(this.monitoringCloudResult[this.id]);
  }

  public closeModal() {
    this.monitoringModal.hide();
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }
}
