import { AlertService, LoadingService, Modal, ModalService, Button, Select, DialogService, Form, Text, Container } from '@zeedhi/common';
import { Config, Http, IEventParam, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { io, Socket } from 'socket.io-client';
import { SocketEvent } from '@/socket/socket-event';
import { CustomPagination } from '@/components/custom-pagination/common/custom-pagination';

export class UpdatesController {
  public socket?: Socket;

  private fullscreenModal!: Modal;

  private logModal!: Modal;

  private prodConfigModal!: Modal;

  private prodConfigEditModal!: Modal;

  private defaultGridTitle!: String;

  private filterProductName!: String;

  private socketArgs = {
    email: 'assistant@teknisa.com',
    password: 'nineninenine',
    slave: false,
  };

  private logRow!: any;

  public async createFullscreenModal() {
    this.fullscreenModal = await ModalService.createFromJson('historico', true);
    this.logModal = await ModalService.createFromJson('updater-log-filial', true);
    this.prodConfigModal = await ModalService.createFromJson('updater-prod-config-modal', true);
    this.prodConfigEditModal = await ModalService.createFromJson('updater-prod-config-edit-modal', true);
  }

  private verifyAndShowUnitStatus(row : any) {
    if (!row.CONNECTION_STATUS) this.showAlert('A conexão com a unidade deve estar ativa para realizar esta ação', '#e68129');
    return row.CONNECTION_STATUS;
  }

  public async openLogBranchModal({ row }: any) {
    if (!this.verifyAndShowUnitStatus(row)) return;
    const params = { CDFILIAL: row.CDFILIAL };
    this.logRow = row;
    this.logModal.show();

    if (params) {
      this.socket?.emit('getUpdaterLog', params);
    }
  }

  public onChangePageSize({ value }: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const allDataLength = (<any>grid.datasource).resultData.length;
    customPagination.page = 1;
    customPagination.length = Math.ceil(allDataLength / value);
    this.onChangePage({ page: 1 });
  }

  public calculatePageNumbers() {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const { value: pageSize } = grid.getChildInstance<Select>('page-size').selectValue;
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const { search } = grid.datasource;
    const {
      filteredData,
      resultData,
    } = (<any>grid.datasource);
    const allDataSize = search ? filteredData.length : resultData.length;
    customPagination.length = Math.ceil(allDataSize / pageSize);
  }

  public onChangePage({ page = 1 }:any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const container = grid.getChildInstance<Container>('page-numbers');
    const customPagination = container.getChildInstance<CustomPagination>('custom-pagination');
    const pageInfo: Text = grid.getChildInstance('page-info');
    const pageSize: Select = grid.getChildInstance('page-size');
    const currentLimit = pageSize.value;
    let { resultData: allData } = (<any>grid.datasource);
    if (grid.datasource.search) {
      allData = (<any>grid.datasource).filteredData;
    }
    const { start, end, data } = this.getPage(allData, page, currentLimit);
    grid.datasource.data = data;
    pageInfo.text = `${start + 1} - ${end} of ${allData.length}`;
    customPagination.page = page;
    grid.buildGroupedData();
  }

  public updaterLogFormBeforeDestroy() {
    this.socket?.emit('stopUpdaterLog', { CDFILIAL: this.logRow.CDFILIAL });
  }

  public updaterLogFormMounted(params: any) {
    const { component: form } = params;
    const divTextArea: any = form.getChildInstance('updater-text-area-result');
    const textArea: any = document.getElementById('updater-text-area-result');
    const textFooter: Text = form.getChildInstance('updater-branch-log');
    textFooter.text += ` ${this.logRow.CDFILIAL}`;
    this.socket?.removeAllListeners('updaterLog');
    this.socket?.on('updaterLog', (param) => {
      if (param.error === false) {
        divTextArea.value += `${param.message}\n`;
        textArea.scrollTop = textArea.scrollHeight;
      }
    });
  }

  public openFullscreenModal({ row }: any) {
    LoadingService.show();

    const filter = {
      params: {
        CDFILIAL: row.CDFILIAL,
        CDPRODUCTID: row.ID,
      },
    };

    Http.get('/updaterProdutoHistorico', filter)
      .then((result) => {
        const grid = this.fullscreenModal.children[1];
        grid.datasource.data = result.data.data;
        this.fullscreenModal.show();
      })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public closeFullscreenModal() {
    this.fullscreenModal.hide();
  }

  public async searchInGrid(searchText: string) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const { resultData } = (<any>grid.datasource);
    grid.datasource.search = searchText;
    (<any>grid.datasource).filteredData = resultData.filter((item: any) => {
      const matchUnit = item.FILIAL.toLowerCase().includes(searchText.toLowerCase());
      const matchVersion = (item.NRVERSAOATUAL || '').includes(searchText);
      return matchUnit || matchVersion;
    });
    this.onChangePage({ page: 1 });
    this.calculatePageNumbers();
    return Promise.resolve((<any>grid.datasource).filteredData);
  }

  public async gridMounted(force: boolean = false) {
    if (!this.socket) {
      this.socket = io(Config.endPoint, {
        transports: ['websocket', 'polling'],
        query: {
          page: 'updates',
        },
      });
    }
    this.socket?.emit(SocketEvent.CONNECTION);
    this.socket?.emit('login', this.socketArgs);
    this.socketListener();
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    grid.setSearch = this.searchInGrid.bind(this);
    const date = new Date().toLocaleDateString();
    const savedDate = localStorage.getItem('LAST_DATE_UPDATER');
    this.defaultGridTitle = grid.title;

    if (savedDate !== date || (typeof force === 'boolean' && force)) {
      LoadingService.show('Atualizando lista de sistemas...');
      const conexao = `${Config.metadataEndPoint}orgProducts`;
      Http.get(conexao)
        .then((result) => {
          if (result.data.isError) {
            this.showAlert(`Erro ao atualizar a lista de sistemas: ${result.data.response}`, 'error');
          } else {
            this.showAlert('Lista de sistemas atualizada com sucesso!', 'success');
          }
        })
        .catch((error) => {
          this.showAlert(error, 'error');
        })
        .finally(() => {
          LoadingService.hide();
          localStorage.setItem('LAST_DATE_UPDATER', date);
          if (typeof force !== 'boolean') {
            this.openFilter(grid);
          }
        });
    } else {
      this.openFilter(grid);
    }
  }

  private openFilter(grid: TekGrid) {
    const filterButton = Metadata.getInstance<Button>(`${grid.name}_filterButton`);
    filterButton.click({} as Event, {} as HTMLElement);
  }

  public updateSystemList(): void {
    this.gridMounted(true);
  }

  private showAlert(message: any, colorStr: string) {
    AlertService.show({
      name: 'info-alert',
      text: message,
      color: colorStr,
      right: true,
      top: true,
      showDismiss: true,
      timeout: 10000,
    });
  }

  public executeDelete(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL,
      INSTALLVERSION: null,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'REMOVE',
    };

    Http.post('/updater/removeVersion', params).then((result) => {
      if (result.data && result.data.error) {
        this.showAlert(result.data.message, 'error');
      } else {
        this.socket?.emit('execUpdater', params);
        this.showAlert('Requisição de remoção enviada com sucesso!', 'success');
      }
    })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public deleteVersion({ row }: any) {
    try {
      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.IDSTEVENTO === 'INSTALACAO_PENDENTE') {
        throw new Error('Sistema não instalado.');
      }

      DialogService.show({
        name: 'remove-dialog',
        type: 'warning',
        title: 'Exclusão',
        text: 'Deseja realmente excluir essa versão desse sistema?',
        buttons: [
          {
            name: 'no',
            component: 'ZdButton',
            label: 'Não',
            outline: true,
            events: {
              click: () => {
                DialogService.hide();
              },
            },
          },
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'Sim',
            events: {
              click: () => {
                DialogService.hide();
                this.executeDelete(row);
              },
            },
          },
        ],
      });
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error.message, '#e68129');
      }
    }
  }

  private async executeInstall(row: any) {
    LoadingService.show();

    const params = {
      CDFILIAL: row.CDFILIAL,
      CDOPERADOR: localStorage.getItem('CDOPERADOR'),
      PRODFILIID: row.PRODFILIID.toString(),
      DEVOPSID: row.ID.toString(),
      CURRENTVERSION: row.NRVERSAOATUAL === '-' ? null : row.NRVERSAOATUAL,
      INSTALLVERSION: row.VERSION.NRVERSAO,
      CREATENEWVERSION: row.IDUTMULTVER,
      TYPE: 'INSTALL',
    };

    Http.post('/updater/installVersion', params)
      .then((result) => {
        if (result.data && result.data.error) {
          this.showAlert(result.data.message, 'error');
        } else {
          this.socket?.emit('execUpdater', params);
          this.showAlert('Requisição de instalação/atualização enviada com sucesso!', 'success');
        }
      })
      .catch((error) => {
        this.showAlert(error, 'error');
      })
      .finally(() => {
        LoadingService.hide();
      });
  }

  public async installVersion({ row }: any) {
    try {
      if (!row.VERSION) {
        return this.showAlert('Selecione a versão', '#e68129');
      }

      if (!this.verifyAndShowUnitStatus(row)) return;

      if (row.VERSION.IDDOWNLOADVER === 'N') {
        throw new Error('Essa versão ainda não está disponível para download. Aguarde alguns instantes e tente novamente.');
      }

      if (row.VERSION.IDDOWNLOADVER === 'E') {
        throw new Error('Ocorreu um erro ao realizar o download dessa versão para a nuvem. Atualize a lista de sistemas, aguarde alguns instantes e tente novamente.');
      }

      const params = {
        CDFILIAL: row.CDFILIAL,
        UPDATERPRODID: row.ID,
        NRVERSAOATUAL: row.VERSION.NRVERSAO,
      };

      const checkVersionInstalled = await Http.post('/updater/checkVersion', params);

      if (checkVersionInstalled.data && checkVersionInstalled.data.error) {
        throw new Error('Essa versão já está instalada.');
      }

      const { data } = await Http.post('/updater/getProdConfig', params);

      if (data && data.error && data.message) {
        throw new Error(data.message);
      }

      if (data && data.message) {
        this.showAlert(data.message, '#e68129');
        await this.validateGetUpdaterProdConfig(data, row);
      } else {
        this.dialog(data.data, row);
      }
    } catch (error) {
      if (error instanceof Error) {
        this.showAlert(error, '#e68129');
      }
    }
  }

  public async validateGetUpdaterProdConfig(data: any, row: any) {
    const footerModal = this.prodConfigModal.children.find((el: any) => el.name === 'updater-prod-config-modal-footer');
    const buttonSaveModal = footerModal?.rightSlot.find((el:any) => el.name === 'save-button-prod-config');
    if (buttonSaveModal && buttonSaveModal.events) {
      buttonSaveModal.events.click = () => {
        this.saveUpdaterProdConfig(row);
      };
    }
    await this.prodConfigModal.show();
    const form: any = Metadata.getInstance<Form>('updater-prod-config-form');
    const iterable: any = form.getChildInstance('IterablecomponentParent');

    const formInputNrVersao: any = this.prodConfigModal.getChildInstance('NRVERSAO');
    formInputNrVersao.value = row.VERSION?.NRVERSAO;

    const formInputCdProductId: any = this.prodConfigModal.getChildInstance('CDPRODUCTID');
    formInputCdProductId.value = row.ID;

    iterable.datasource.data = [];

    Object.entries(data.data).forEach(([key, element]) => {
      const productConfigObject = {
        id: key,
        value: element,
        label: key,
      };
      if (productConfigObject) {
        iterable.datasource.data.push(productConfigObject);
      }
    });
  }

  public async saveUpdaterProdConfig(rowGrid: any) {
    try {
      const form: any = Metadata.getInstance<Form>('updater-prod-config-form');
      const nrVersao: any = this.prodConfigModal.getChildInstance('NRVERSAO');
      const cdProductId: any = this.prodConfigModal.getChildInstance('CDPRODUCTID');
      const iterable: any = form.getChildInstance('IterablecomponentParent');
      const inputs = [...form.element].filter((item) => item.type === 'text');
      const obj: any = {};

      if (inputs && iterable) {
        inputs.forEach((row: any) => {
          iterable.datasource.data.forEach((item: any) => {
            if (row.id === item.id) {
              obj[item.label] = row.value;
            }
          });
        });
      }

      const params = {
        CDPRODUCTID: cdProductId.value,
        NRVERSAO: nrVersao.value,
        CONFIG: obj,
      };
      const { data } = await Http.post('/updater/saveProdConfig', params);

      if (data && data.error) {
        throw new Error(data.message);
      }

      this.showAlert(data.message, 'success');
      this.prodConfigModal.hide();
      this.dialog(data.data, rowGrid);
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  private isEmpty(item: any) {
    if (!item) return true;
    if (Array.isArray(item) && !item.length) return true;
    if (typeof item === 'object' && !Object.keys(item).length) return true;
    return false;
  }

  public async createProdConfigEditModal({ row }: any) {
    try {
      if (!row.VERSION) {
        return this.showAlert('Selecione a versão', '#e68129');
      }

      const params = {
        UPDATERPRODID: row.ID,
        NRVERSAOATUAL: row.VERSION.NRVERSAO,
      };

      const { data } = await Http.post('/updater/getProdConfig', params);

      if (data && data.error && data.message) {
        throw new Error(data.message);
      }

      if (data && data.message) {
        this.showAlert(data.message, '#e68129');
        await this.validateGetUpdaterProdConfig(data, row);
      } else if (this.isEmpty(data.data)) {
        this.showAlert('Projeto/versão não possui parâmetros', '#e68129');
      } else {
        await this.prodConfigEditModal.show();
        const form: any = Metadata.getInstance<Form>('updater-prod-config-edit-form');
        const iterable: any = form.getChildInstance('IterablecomponentParent');

        iterable.datasource.data = [];
        const formInputNrVersaoEdit: any = this.prodConfigEditModal.getChildInstance('NRVERSAO');
        formInputNrVersaoEdit.value = row.VERSION?.NRVERSAO;
        const formInputCdProductIdEdit: any = this.prodConfigEditModal.getChildInstance('CDPRODUCTID');
        formInputCdProductIdEdit.value = row.ID;

        Object.entries(data.data).forEach(([key, element]) => {
          const productConfigParametersObject = {
            id: `IterablecomponentParent-${key}`,
            value: element,
            label: key,
          };
          if (productConfigParametersObject) {
            iterable.datasource.data.push(productConfigParametersObject);
          }
        });
      }
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  public async saveUpdaterProdConfigEdit() {
    try {
      const form: any = Metadata.getInstance<Form>('updater-prod-config-edit-form');
      const iterable: any = form.getChildInstance('IterablecomponentParent');
      const inputs = [...form.element].filter((item) => item.type === 'text');

      const formInputNrVersaoEdit: any = this.prodConfigEditModal.getChildInstance('NRVERSAO');
      const formInputCdProductIdEdit: any = this.prodConfigEditModal.getChildInstance('CDPRODUCTID');
      const obj: any = {};

      if (inputs && iterable) {
        inputs.forEach((row: any) => {
          iterable.datasource.data.forEach((item: any) => {
            if (row.id === item.id) {
              obj[item.label] = row.value;
            }
          });
        });
      }

      const params = {
        CDPRODUCTID: formInputCdProductIdEdit.value,
        NRVERSAO: formInputNrVersaoEdit.value,
        CONFIG: obj,
      };

      const { data } = await Http.post('/updater/saveProdConfig', params);

      if (data && data.error) {
        throw new Error(data.message);
      }

      this.showAlert(data.message, 'success');
      this.closeUpdaterProdConfigEdit();
    } catch (error) {
      this.showAlert(error, '#e68129');
    }
  }

  private dialog(data: any, row?: any): void {
    let prodConfigValues = '';

    Object.entries(data).forEach(([key, value]) => {
      prodConfigValues += `<strong>${key}: ${value}</strong><br>`;
    });
    const hasParameters = !!prodConfigValues;
    const textDialogPrefix = hasParameters ? `<p>Parâmetros do Projeto</p>\n${prodConfigValues}` : '<strong>Projeto não possui parâmetros</strong>';
    DialogService.show({
      name: 'install-dialog',
      type: 'warning',
      title: 'Instalação',
      text: `    
      ${textDialogPrefix}
      <br>Deseja realmente instalar/atualizar esse sistema?                      
        `,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          outline: true,
          events: {
            click: () => {
              DialogService.hide();
            },
          },
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => {
              DialogService.hide();
              this.executeInstall(row);
            },
          },
        },
      ],
    });
  }

  private async socketListener() {
    this.socket?.on('updaterInProgress', (result) => {
      const messages: any = {
        INSTALACAO_EM_CURSO: 'Instalação iniciada...',
        ATUALIZACAO_EM_CURSO: 'Atualização iniciada...',
        REMOCAO_EM_CURSO: 'Remoção iniciada...',
      };
      let messageCode = '';
      switch (result.operation) {
        case 'INSTALL':
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
        case 'UPDATE':
          messageCode = 'ATUALIZACAO_EM_CURSO';
          break;
        case 'REMOVE':
          messageCode = 'REMOCAO_EM_CURSO';
          break;
        default:
          messageCode = 'INSTALACAO_EM_CURSO';
          break;
      }
      this.showAlert(messages[messageCode], 'success');
      this.updateGridRowByUnitOrId(result, { IDSTEVENTO: messageCode });
    });

    this.socket?.on('updaterFinish', (result) => {
      if (result.error) {
        this.showAlert('Ocorreu um erro durante a execução do último processo. Verifique o histórico para mais informações.', 'error');
        this.updateGridRowByUnitOrId(result, { NRVERSAOATUAL: result.VERSION, IDSTEVENTO: null });
      } else {
        this.showAlert('Operação finalizada com sucesso!', 'success');
        const availableVersion = Metadata.getInstance<Select>(`availableVersion_${result.PRODFILIID}`);
        const IDSTEVENTO = result.operation === 'REMOVE' ? 'INSTALACAO_PENDENTE' : null;
        availableVersion.setValue(null);
        this.updateGridRowByUnitOrId(result, { NRVERSAOATUAL: result.VERSION, IDSTEVENTO });
      }
    });
    this.socket?.on('disconnect', () => {
      this.showAlert('A página será recarregada para restabelecer a conexão...', 'error');
      setTimeout(() => {
        document.location.reload();
      }, 5000);
    });

    this.socket?.on('updaterConnected', async ({ UNITS, CONNECTED }) => {
      const grid = Metadata.getInstance<TekGrid>('updater-grid');
      let updatedUnits = 0;
      const { resultData = [] } = (<any>grid.datasource);
      resultData.every((unit: any) => {
        if (UNITS.includes(unit.CDFILIAL)) {
          Object.assign(unit, { CONNECTION_STATUS: CONNECTED });
          updatedUnits += 1;
          if (UNITS.length === updatedUnits) return false;
        }
        return true;
      });
      grid.buildGroupedData();
    });
  }

  public updateGridRowByUnitOrId({ CDFILIAL, PRODFILIID }: any, newValues: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const { resultData } = (<any>grid.datasource);
    let matchResult = false;
    for (let i = 0; i < resultData.length; i += 1) {
      const curItem = resultData[i];
      if ((CDFILIAL && curItem.CDFILIAL === CDFILIAL) || (PRODFILIID && PRODFILIID.toString() === curItem.PRODFILIID.toString())) {
        matchResult = true;
        Object.assign(curItem, newValues);
      } else if (matchResult) {
        break;
      }
    }
    grid.buildGroupedData();
  }

  public setAvailableVersionFilter({ component }: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const [firstRow] = grid.datasource.data;
    const { ID } = firstRow;
    component.datasource.filter = {
      CDPRODUCTID: ID,
    };
  }

  public async beforeApplyFilter({ component, event }: any) {
    // Force to reload all available version column
    // component.datasource.updateData([]);
    event.preventDefault();
    const [productIdComponent] = component.getFilterInputs();
    const { value: PRODUCT_ID } = productIdComponent;
    this.updateGridData(PRODUCT_ID);
    ModalService.closeByName('updater-grid-filter-modal');
    component.toolbarSlot[0].text = `${this.defaultGridTitle} - ${this.filterProductName}`;
  }

  private getPage(dataset: any, page: number, limit: number) {
    const end: number = page * limit;
    const start = end - limit;
    const data = dataset.slice(end - limit, end);
    return {
      start,
      end: end > dataset.length ? dataset.length : end,
      data,
    };
  }

  public async updateGridData(PRODUCT_ID: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    const result = await Http.get('/updaterProdutosFilial', { params: { PRODUCT_ID } }).catch((err) => {
      console.error(err);
    });
    (<any>grid.datasource).resultData = result.data.data;
    const { limit } = grid.datasource;
    const container: Container = grid.getChildInstance<Container>('page-numbers');
    const customPagination : CustomPagination = container.getChildInstance('custom-pagination');
    customPagination.length = Math.ceil(result.data.data.length / limit);
    this.onChangePage({ page: 1 });
  }

  public changeProductId({ component, value }: any) {
    const data = component.datasource.data.find((item: any) => item.ID === value);
    if (data) {
      this.filterProductName = data.NMPRODUCT;
    }
  }

  public gridBeforeDestroy() {
    this.socket?.removeAllListeners('updaterInProgress');
    this.socket?.removeAllListeners('updaterFinish');
    this.socket?.removeAllListeners('updaterConnected');
    this.socket?.removeAllListeners('disconnect');
    this.fullscreenModal?.destroy();
    this.logModal?.destroy();
    this.prodConfigModal.destroy();
    this.prodConfigEditModal.destroy();
    ModalService.getModal('updater-grid-filter-modal')?.destroy();
  }

  public closeProdConfigModal(): void {
    this.prodConfigModal.hide();
  }

  public closeUpdaterProdConfigEdit(): void {
    this.prodConfigEditModal.hide();
  }

  public getConnectionColor({ row }: any) {
    return row.CONNECTION_STATUS ? 'green' : 'red';
  }

  public changeSelectVersion({ value }: IEventParam<Select>) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    grid.datasource.currentRow.VERSION = value;
  }

  public beforeOpenReport(reportParams: any) {
    const grid = Metadata.getInstance<TekGrid>('updater-grid');
    reportParams.dataSet = (<any>grid.datasource).resultData;
    reportParams.dataSet = JSON.parse(JSON.stringify(reportParams.dataSet));
    reportParams.metaData.filter = [{ label: 'Produto', value: this.filterProductName }];
    const { componentProps: { datasource, dataValue, dataText } } = grid.getColumn('IDSTEVENTO');
    const events = datasource.data;
    const messageEvents = events.reduce((acc: any, cv: any) => {
      acc[cv[dataValue]] = cv[dataText];
      return acc;
    }, {});
    reportParams.dataSet.forEach((item: any) => {
      item.IDSTEVENTO = messageEvents[item.IDSTEVENTO] || item.IDSTEVENTO;
    });
    return reportParams;
  }
}
